package de.htwberlin.magw;



import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import de.htwberlin.magw.helper.Debug;
import de.htwberlin.magw.model.ArztInfo;
import de.htwberlin.magw.session.SessionManager;

/* */
public class MAGWmobileActivity extends NfcPendingIntentActivity {

	private static final String LOG_TAG = "MAGWmobileActivity";
	EditText et_password;

	/** Called when the activity is first created. */
	/*
	 * TODO Neue Activity-Klasse von Activity ableiten und implementieren der
	 * Pedning-Intent-Funktionalitt zum Abfangen der NFC-Intents Alle
	 * Activities des Projektes erben von neuer Activity-Klasse
	 */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);
		Log.d(LOG_TAG, "onCreate");

		this.et_password = (EditText)findViewById(R.id.et_password);
		Button login = (Button)findViewById(R.id.bt_login);
		login.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				String password = et_password.getText().toString();
				// TODO create hash of password
				String hash = password;
				loginUser(hash);
			}
		});

		Button debug = (Button) findViewById(R.id.bt_debug);
		this.setDebugButtonText();
		final  Intent home_intent = new Intent(this, de.htwberlin.magw.HomeActivity.class);
		debug.setOnClickListener(new View.OnClickListener() {
			public void onClick(View view) {
				if (Debug.getInstance().isDebugFlag() == true) {
					Debug.getInstance().setDebugFlag(false);
				} else {
					Debug.getInstance().setDebugFlag(true);
					startActivity(home_intent);
				}
				setDebugButtonText();
			}
		});
		
		Button logout = (Button) findViewById(R.id.bt_logout);        
		logout.setOnClickListener(new View.OnClickListener() {
			public void onClick(View view) {
				SessionManager session = SessionManager.getInstance();
				ArztInfo ai = ArztInfo.getInstance();
				session.logoutUser();
				ai.reset();
				setCurrentUserName();
				changeVisibilityOfLogoutButton(false);
			}
		});
		
		this.setCurrentUserName();
	}
	
	@Override
	public void onRestart() {
		super.onRestart();
		Log.d(LOG_TAG, "OnRestart");
		if (et_password.getText().toString() != null && !et_password.getText().toString().equalsIgnoreCase("")) {
			et_password.setText("");
		}
		this.setCurrentUserName();
	}
	
	@Override
	public void onNewIntent(Intent intent) {
		super.onNewIntent(intent);
		Log.d(LOG_TAG, "OnNewIntent");
		if (intent.hasExtra("LOGOUT")) {
			Log.d(LOG_TAG, "Extra LOGOUT given");
			this.changeVisibilityOfLogoutButton(intent.getExtras().getBoolean("LOGOUT"));
		} else {
			Log.d(LOG_TAG, "Extra LOGOUT not given");
			this.changeVisibilityOfLogoutButton(false);
		}
		this.setCurrentUserName();
	}

	private void loginUser(String hashedPassword) {
		if (this.session.loginUser(ArztInfo.getInstance().getUserId(), ArztInfo.getInstance().getPassword(), hashedPassword)) {
			Intent intent = new Intent(this, HomeActivity.class);
			startActivity(intent);
		} else {
			this.postToastMessage("Login failed.");
		}
	}
	
	private void setCurrentUserName() {
		ArztInfo ai = ArztInfo.getInstance();
		TextView currentUser = (TextView)findViewById(R.id.current_user);
		if (ai.getUserId() != null) {
			String name = ai.getName() + " " + ai.getLastname();
			currentUser.setText(name);
		} else {
			currentUser.setText("");
		}
	}
	
	private void changeVisibilityOfLogoutButton(boolean visible) {
		Button logout = (Button)findViewById(R.id.bt_logout);
		if (visible == true) {
			logout.setVisibility(View.VISIBLE);
		} else {
			logout.setVisibility(View.INVISIBLE);
		}
	}
	
	private void setDebugButtonText() {
		Button debug = (Button) findViewById(R.id.bt_debug);
		if (Debug.getInstance().isDebugFlag() == true) {
			// set Text to debug_on, when debugging is on
			debug.setText(R.string.debug_on);
		} else {
			// set Text to debug_off, when debugging is off
			debug.setText(R.string.debug_off);
		}
	}
}

