package de.htwberlin.magw;

import java.util.ArrayList;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.app.ListActivity;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import de.htwberlin.magw.helper.JsonUrlLoader;
import de.htwberlin.magw.model.DiagnosticReport;
import de.htwberlin.magw.model.PatientInfo;

public class MedicalHistoryActivity extends ListActivity{

	private static final String LOGTAG = "MedicalHistoryActivity";
	private static MedicalHistoryActivity activity;
	
	private ArrayList<DiagnosticReport> reports;
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		activity = this;
		Log.d(LOGTAG, "onCreate");
		// TODO create layout for medical history
		//setContentView(R.layout.history)
		this.reports = new ArrayList<DiagnosticReport>();
		this.getDiagnosticReports();
		String[] entries = new String[reports.size()];
		for (int i=0; i<reports.size(); i++) {
			entries[i] = reports.get(i).getDate().substring(0, 10) + " " + reports.get(i).getTitle();
		}
		setListAdapter(new ArrayAdapter<String>(this, R.layout.history_list, entries));
		
		ListView listView = getListView();
		listView.setTextFilterEnabled(true);
		listView.setOnItemClickListener(new OnItemClickListener() {
			public void onItemClick(AdapterView<?> parent, View view,
					int position, long id) {
				// When clicked, show a toast with the TextView text
				//Toast.makeText(getApplicationContext(),
				//		((TextView) view).getText(), Toast.LENGTH_SHORT).show();
				Intent intent = new Intent(activity, DiagnosticReportActivity.class);
				intent.putExtra("TITLE", reports.get((int)id).getTitle());
				intent.putExtra("TEXT", reports.get((int)id).getText());
				intent.putExtra("IMAGE", reports.get((int)id).getImageUrl());
				intent.putExtra("DATE", reports.get((int)id).getDate());
				intent.addFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP);
				activity.startDiagnosticReportActivity(intent);
			}
		});
	
	}
	
	public void startDiagnosticReportActivity(Intent intent) {
		this.startActivity(intent);
	}
	
	@Override
	public void onRestart() {
		super.onRestart();
		Log.d(LOGTAG, "onRestart");
	}

	@Override
	public void onNewIntent(Intent intent) {
		super.onNewIntent(intent);
		Log.d(LOGTAG, "onNewIntent");
	}
	
	private void getDiagnosticReports() {
		// load data for current patient from backend to parse IDs of diagnostic reports
		String url = "http://mgt.inka.f4.htw-berlin.de:5000/personen/" + PatientInfo.getInstance().getUserId();
		JSONObject json = JsonUrlLoader.loadJsonFromUrl(url);
		try {
			JSONArray indications = json.getJSONArray("befunde");
			Log.d(LOGTAG, "Anzahl Befunde: " + indications.length());
			for (int i=0; i<indications.length(); i++) {
				String indication_id = indications.getString(i);
				// load specific diagnostic report
				String indication_url = "http://mgt.inka.f4.htw-berlin.de:5000/befunde/" + indication_id;
				DiagnosticReport report = this.loadDiagnosticReportsFromUrl(indication_url);
				if (report != null) {
					this.reports.add(report);
				}
			}
		} catch (JSONException e) {
			Log.d(LOGTAG, "Parsing error: ");
			e.printStackTrace();
		}
	}
	
	private DiagnosticReport loadDiagnosticReportsFromUrl(String url) {
		JSONObject json = JsonUrlLoader.loadJsonFromUrl(url);
		try {
			String id = json.getString("id");
			String img_url = json.getString("bild");
			String title = json.getString("titel");
			String text = json.getString("text");
			String person = json.getString("person");
			String date = json.getString("datum");
			Log.d(LOGTAG, "Report data:" + date + " " + title);
			DiagnosticReport report = new DiagnosticReport(id, person, title, text, img_url, date);
			return report;
		} catch (JSONException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}
}
