package de.htwberlin.magw;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.LinkedList;
import java.util.SortedSet;
import java.util.TreeSet;
import android.app.Activity;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.nfc.NdefMessage;
import android.nfc.NdefRecord;
import android.nfc.NfcAdapter;
import android.nfc.Tag;
import android.nfc.tech.Ndef;
import android.nfc.tech.NfcA;
import android.os.Bundle;
import android.os.Parcelable;
import android.util.Log;
import android.widget.Toast;
import de.htwberlin.magw.helper.Debug;
import de.htwberlin.magw.model.ArztInfo;
import de.htwberlin.magw.model.PatientInfo;
import de.htwberlin.magw.model.VitalDataSet;
import de.htwberlin.magw.session.SessionManager;

public abstract class NfcPendingIntentActivity extends Activity {

	private static final String LOG_TAG = "NfcPendingIntentActivity";
	protected NfcAdapter mNfcAdapter;
	protected PendingIntent pendingIntent;
	private String[][] mTechList;
	protected SessionManager session;
	
	protected BroadcastReceiver mReceiver = new BroadcastReceiver (){

		@Override
		public void onReceive(Context context, Intent intent)
		{
			if (intent.getAction().equals(Intent.ACTION_SCREEN_OFF))
			{
				Log.v(LOG_TAG, "ACTION_SCREEN_ON");
				session.setScreenLock(true);
				// onPause() will be called.
	            }
			else if (intent.getAction().equals(Intent.ACTION_SCREEN_ON))
			{
				Log.v(LOG_TAG, "ACTION_SCREEN_ON");
				//onResume() will be called.
				//Better check for whether the screen was already locked
				// if locked, do not take any resuming action in onResume()
				//Suggest you, not to take any resuming action here.       
			}
			else if(intent.getAction().equals(Intent.ACTION_USER_PRESENT))
			{
				Log.v("$$$$$$", "In Method:  ACTION_USER_PRESENT");
				//Handle resuming events
			}
		}
	};
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		// get SessionManager instance
		session = SessionManager.getInstance();
		this.registerBroadcastReceiver();
		// Initialize the NFC adapter
		mNfcAdapter = NfcAdapter.getDefaultAdapter(this);
		if (mNfcAdapter != null) {
			Log.d(LOG_TAG, "NFC adapter available");
		} else {
			Log.d(LOG_TAG, "NFC adapter unavailable");
		}
		// Create the PendingIntent object which will contain the details of the tag that has been scanned
		pendingIntent = PendingIntent.getActivity(this, 0, new Intent(this, getClass()).addFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP), 0);
		Log.d(LOG_TAG, getClass().getName());
		// Setup a tech list for all desired tag types
		mTechList = new String[][] { new String[] { NfcA.class.getName() } };
	}
	
	/** Re-enable the tag dispatch if the app is in the foreground */
	@Override
    public void onResume() {
        super.onResume();
        if (mNfcAdapter != null) mNfcAdapter.enableForegroundDispatch(this, pendingIntent, null, mTechList);
        this.registerBroadcastReceiver();
    }
	
	/** Disable the tag dispatch when the app is no longer in the foreground */
    @Override
    public void onPause() {
 	   super.onPause();
 	   unregisterReceiver(mReceiver);
 	   if (mNfcAdapter != null) mNfcAdapter.disableForegroundDispatch(this);
    }

    /** A tag has been discovered */
    @Override 
    public void onNewIntent(Intent intent) {
    	super.onNewIntent(intent);
    	Log.d(LOG_TAG, "OnNewIntent");
    	if (intent.getAction() != null && intent.getAction().equalsIgnoreCase("android.nfc.action.TECH_DISCOVERED")) {
	    	//get ID
	    	Tag tagFromIntent = intent.getParcelableExtra(NfcAdapter.EXTRA_TAG);
	    	// if onNewIntent was called by another event than NFC
	    	if (tagFromIntent != null) {
		    	byte id[] = tagFromIntent.getId();  
		    	String uidS = getHexString(id);
		    	Log.d(LOG_TAG, uidS);
				Ndef ndef = Ndef.get(tagFromIntent);
				if (ndef != null) {
					try {
						ndef.connect();
					} catch (IOException e) {
						e.printStackTrace();
						Log.d(LOG_TAG, "konnte nicht connecten");
					}
					Log.d(LOG_TAG, String.valueOf(ndef.getMaxSize()));
				} else {
					Log.d(LOG_TAG, "hat nichts geholt");
				}
				
				//get ndef records
				
				Parcelable[] rawMsgs = intent.getParcelableArrayExtra(NfcAdapter.EXTRA_NDEF_MESSAGES);
				NdefMessage[] msgs;
				if (rawMsgs != null) {
					msgs = new NdefMessage[rawMsgs.length];
					for (int i = 0; i < rawMsgs.length; i++) {
						msgs[i] = (NdefMessage) rawMsgs[i];
						Log.d(LOG_TAG, "karte hat ndef messages");
					}
				} else {
					// Unknown tag type
					byte[] empty = new byte[] {};
					NdefRecord record = new NdefRecord(NdefRecord.TNF_UNKNOWN, empty, empty, empty);
					NdefMessage msg = new NdefMessage(new NdefRecord[] {record});
					msgs = new NdefMessage[] {msg};
				}
				final int size = msgs.length;
				LinkedList<String> nfcContent = new LinkedList<String>();
				for (int i = 0; i < size; i++) {
					NdefRecord[] ndefRec = msgs[i].getRecords();
					for(NdefRecord r : ndefRec){
						// if(r.getTnf() == NdefRecord.TNF_WELL_KNOWN && r.getType().equals(NdefRecord.RTD_TEXT)){
						//Log.d(LOG_TAG, "im record parser");
						try {	
							byte[] payload = r.getPayload();
							String textEncoding = ((payload[0] & 0200) == 0) ? "UTF-8" : "UTF-16";
							int languageCodeLength = payload[0] & 0077;
							String text =
									new String(payload, languageCodeLength + 1,
											payload.length - languageCodeLength - 1, textEncoding);
							//Log.d(LOG_TAG, text);
							//Log.d(LOG_TAG, String.valueOf(text.length()));
							nfcContent.add(text);
						} catch (UnsupportedEncodingException e) {
							// should never happen unless we get a malformed tag.
							throw new IllegalArgumentException(e);
						}
					}
				}
			
				if(nfcContent.get(0).equals("P")){
					// Patient
					Log.d(LOG_TAG, "ha ist Patienten Karte");
					SessionManager session = SessionManager.getInstance();
					if (session.getLoggedInUser() != null || Debug.getInstance().isDebugFlag() == true) {
						//get Patient
						PatientInfo patient = PatientInfo.getInstance();
						//Backend ID
						patient.setUserId(nfcContent.get(2));
						//Bloodgroup
						patient.setBloodgroup(nfcContent.get(6));
						//Name
						String name = nfcContent.get(3);
						String[] names= name.split(", ");
						patient.setLastname(names[0]);
						patient.setName(names[1]);
						//Nofallnummer
						patient.setTelephone(nfcContent.get(4));
						// parse allergies
						String[] allergies = nfcContent.get(5).split(", ");
						SortedSet<String> allergySet = new TreeSet<String>();
						for (int i=0; i<allergies.length; i++) {
							if (allergies[i] != null && !allergies[i].equalsIgnoreCase("")) {
								allergySet.add(allergies[i]);
							}
						}
						patient.setAllergien(allergySet);
						//parse Vital Daten
						//parse Vital Daten
						String pulsCitalString = nfcContent.get(8);
						if(pulsCitalString.length()%14!=0){
							Log.d(LOG_TAG,"vital daten string nicht richtig");
						}else{
							SortedSet<VitalDataSet> vitalSet = new TreeSet<VitalDataSet>();
							int anzVitalSets = (int)pulsCitalString.length()/14;
							int startPos=0;
							while(anzVitalSets!=0){
								VitalDataSet vital = new VitalDataSet(pulsCitalString.substring(startPos,startPos+14));
								vitalSet.add(vital);
								startPos+=14;
								anzVitalSets--;
							}
							patient.setPulsVitalDataSet(vitalSet);
						}
							String tempVitalDatenSet = nfcContent.get(9);
							if(tempVitalDatenSet.length()%14!=0){
								Log.d(LOG_TAG,"vital daten string nicht richtig");
							}else{
								SortedSet<VitalDataSet> vitalTempSet = new TreeSet<VitalDataSet>();
								 int anzVitalSets = (int)tempVitalDatenSet.length()/14;
								 int startPos=0;
								while(anzVitalSets!=0){
									VitalDataSet vital = new VitalDataSet(tempVitalDatenSet.substring(startPos,startPos+14));
									vitalTempSet.add(vital);
									startPos+=14;
									anzVitalSets--;
								}
								patient.setTempVitalDataSet(vitalTempSet);
							
							/*for (VitalDataSet vitalDataSet : vitalSet) {
								Log.d("test", vitalDataSet.getDate().toGMTString());
								Log.d("test", "min value: "+vitalDataSet.getMinValue());
								Log.d("test", "max value: "+vitalDataSet.getMaxValue());
								Log.d("test", "avr value: "+vitalDataSet.getAvrValue());
							}*/
						} 
			
						//start activity
			    		this.startVitaldatenActivity();
					} else {
						this.postToastMessage(this.getString(R.string.permission_denied));
					}
				} else if (nfcContent.get(0).equals("A")) {
					// Anmeldung Arzt
					// ArztInfo Singleton erstellen, ID, Name und Passwort extrahieren
					String aID = nfcContent.get(2);
					String aPassWd = nfcContent.get(1);
					String[] name = nfcContent.get(3).split(", ");
					String aForeName = name[1];
					String aLastName = name[0];
					ArztInfo aInfo = ArztInfo.getInstance();
					SessionManager session = SessionManager.getInstance();
					// check if userId of NFC card and ArztInfo object are the same
					if (aInfo.getUserId() != null && aInfo.getUserId().equalsIgnoreCase(aID)) {
						// values of ArztInfo objects should be up to date
						//check session and do re-login if necessary
						if (session.isUserLoggedIn(aInfo.getUserId()) == false) {
							// re-login, start LoginActivity
							this.startLoginActivity(false);
						} else {
							// reset last login time and start LoginActivity with logout button
							session.resetLastLogin();
							this.startLoginActivity(true);
						}
					} else {
						// logout current user, set values of ArztInfo object to current user and start LoginActivity
						session.logoutUser();
						aInfo.setUserId(aID);
						aInfo.setName(aForeName);
						aInfo.setLastname(aLastName);
						aInfo.setPassword(aPassWd);
						this.startLoginActivity(false);
					}
				}
    		}
    	}
    }
    
    private void startVitaldatenActivity(){
    	Intent intentPatient = new Intent(this, de.htwberlin.magw.VitaldatenActivity.class);
    	intentPatient.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
		startActivity(intentPatient);
		//remove caller from activity stack
		//this.finish();
    }
    
    private void startLoginActivity(boolean withLogoutButton) {
    	if (!withLogoutButton) {
			this.postToastMessage(this.getString(R.string.authetication_card_accepted));
		} else {
			this.postToastMessage(this.getString(R.string.authentication_or_logout));
		}
    	if (!this.getClass().getCanonicalName().equalsIgnoreCase("de.htwberlin.magw.MAGWmobileActivity")) {
    		Intent loginIntent = new Intent(this, de.htwberlin.magw.MAGWmobileActivity.class);
    		loginIntent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
    		loginIntent.putExtra("LOGOUT", withLogoutButton);
    		startActivity(loginIntent);
    		//this.finish();
    	}
    }
    
    private static String getHexString(byte[] b) {
    	String result = "";
    	for (int i=0; i < b.length; i++) {
    		result += Integer.toString( ( b[i] & 0xff ) + 0x100, 16).substring( 1 );
    	}
    	return result;
    }
    
    private void registerBroadcastReceiver() {
    	// register BroadcastReceiver
    			IntentFilter filter = new IntentFilter(Intent.ACTION_SCREEN_OFF);
    			filter.addAction(Intent.ACTION_SCREEN_ON);
    			filter.addAction(Intent.ACTION_USER_PRESENT);
    			registerReceiver(mReceiver, filter);
    }
    
    protected void postToastMessage(String message) {
    	Toast notification = Toast.makeText(this, message, Toast.LENGTH_LONG);
		notification.show();
    }
}
