package de.htwberlin.magw.session;

import java.util.Date;

import de.htwberlin.magw.model.ArztInfo;
import de.htwberlin.magw.model.PatientInfo;

public class SessionManager {
	
	private static SessionManager _instance;
	
	private String loggedInUser;
	private Date lastLogin;
	private boolean screenLock;
	
	
	private SessionManager() {
		this.loggedInUser = null;
		this.lastLogin = null;
		this.screenLock = false;
	}
	
	public static SessionManager getInstance() {
		if (_instance == null) {
			_instance = new SessionManager();
		}
		return _instance;
	}
	
	public void clearInstance() {
		this.loggedInUser = null;
		this.lastLogin = null;
		this.screenLock = false;
	}
	
	public boolean loginUser(String userId, String nfcPasswdHash, String typedPasswdHash) {
		// compare typed in passwd with nfc passwd, logout previously logged in user
		if (userId != null && nfcPasswdHash != null && typedPasswdHash != null 
				&& nfcPasswdHash.equalsIgnoreCase(typedPasswdHash)) {
			this.loggedInUser = userId;
			this.lastLogin = new Date();
			this.screenLock = false;
			return true;
		}
		return false;
	}
	
	public void resetLastLogin() {
		this.lastLogin = null;
		this.lastLogin = new Date();
	}
	
	public boolean logoutUser() {
		// logout user
		this.loggedInUser = null;
		this.lastLogin = null;
		this.screenLock = false;
		PatientInfo.getInstance().reset();
		ArztInfo.getInstance().reset();
		return true;
	}
	
	public String getLoggedInUser() {
		// return userId of current logged in user
		return this.loggedInUser;
	}
	
	public boolean isUserLoggedIn(String userId) {
		if (this.loggedInUser != null && this.loggedInUser.equalsIgnoreCase(userId)) {
			Date threshold = new Date();
			long diff = threshold.getTime() - 600000;
			threshold.setTime(diff);
			if (this.loggedInUser != "" && this.lastLogin.after(threshold)) {
				return true;
			}
		}
		return false;
	}
	
	public boolean wasScreenLocked() {
		return this.screenLock;
	}
	
	public void setScreenLock(boolean lock) {
		this.screenLock = lock;
	}
}
