/*
 * TODO: <Classname> - <Description> 
 *
 * Copyright (c) 2011 Stephan Bergemann oberling@deroberling.de
 *
 * This file is part of TODO: <Projectname>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program (see COPYING); if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA.
 *
 */


#include "nfccommunicator.h"
#include "stdlib.h"
#include "nfcrfid/MifareDesfire.h"
#include <QDateTime>

#include <QTextStream>
#include <QtEndian>
#include <QVector>
#include <QDataStream>

NFCCommunicator::NFCCommunicator()
{
    _NFCManager = new LibnfcDeviceManager();
    if(!_NFCManager->connect()) {
        QTextStream cerr(stderr, QIODevice::WriteOnly);
        cerr<<"kann nicht zu NFC verbinden"<<endl;
    }
}

void NFCCommunicator::writeData(QMap<QString, quint8> data, quint8 record) {
    bool isPatient=false;
    QTextStream cout(stdout, QIODevice::WriteOnly);
    std::vector<Transponder> stdtransponder = _NFCManager->readTransponders();
    cout<<"writedata: habe "<<stdtransponder.size()<<" transponder gesehen!"<<endl;
    if (stdtransponder.size() == 1 ){
        NDEFMessage message = _NFCManager->readNDEFMessage(&stdtransponder[0]);
        if(stdtransponder[0].getType() == DESFIRE_MIFARE) {
            std::vector<NDEFRecord*> records = message.getNdefRecords();
            if(records.size()!=0) {
                cout<<"habe folgendes gelesen: "<<endl;
                for(uint i=0; i<records.size(); i++) {
                    cout<<" "<<i<<": "<<QString::fromStdString(records[i]->getPayload())<<endl;
                    if(i==0) {
                        QString person = QString::fromStdString(records[i]->getPayload());
                        if(person.compare("P")==0) {
                            isPatient = true;
                            cout<<"ist ein Patient!"<<endl;
                        }
                    } else if(i==record) {
                        if(isPatient) {
                            QByteArray olddata = QByteArray::fromHex(QString::fromStdString(records[i]->getPayload()).toAscii());
                            quint16 position = 0;
                            quint32 lastdate = 0;
                            quint16 d=0;
                            for(; d<olddata.size(); d+=7) {
                                quint32 thisdata;
                                QDataStream datestream(olddata.mid(d,4));
                                datestream >> thisdata;
                                position=d;
                                if(thisdata<lastdate) {
                                    break;
                                }
                                lastdate=thisdata;
                            }
                            if(d>20*7) { // TODO: 20 ist zu niedrig - bis 140 ist möglich für 3 Sensoren!
                                d=0; // dann ist der buffer für diesen Sensor voll
                            }
                            cout<<"schreibe an Position "<<d/7<<endl;
                            quint32 now = QDateTime::currentDateTime().toTime_t();
                            now = qToBigEndian<quint32>(now);
                            olddata.replace(d, 4, (char*) &now, 4);
                            olddata.replace(d+4, 1,(char*) &data["min"], 1);
                            olddata.replace(d+5, 1,(char*) &data["max"], 1);
                            olddata.replace(d+6, 1,(char*) &data["avg"], 1);
                            olddata = olddata.toHex();
//                            cout<<" schreibe "<<QString(olddata)<<endl;
                            records[i]->setPayload(QString(olddata).toStdString());
                            _NFCManager->writeNDEFMessage(&stdtransponder[0], &message);
                        }
                    }
                }
            } else {
                cout<<"Dieser Transponder enthält keine NDEF-Records!"<<endl;
            }
        } else {
            cout<<"Dieser Transponder ist kein DESFIRE-Transponder!"<<endl;
        }
    } else {
        cout<<"Bitte nur einen Transponder auf das Bett legen!"<<endl;
    }
}

QVector<Transponder> NFCCommunicator::readTransponders() {
    QTextStream cout(stdout, QIODevice::WriteOnly);
    std::vector<Transponder> stdtransponder = _NFCManager->readTransponders();
    cout<<"habe "<<stdtransponder.size()<<" transponder gesehen!"<<endl;
    QVector<Transponder> transponder = QVector<Transponder>::fromStdVector(stdtransponder);
    return transponder;
}

QVector<quint8> NFCCommunicator::getBedData() {
    QVector<quint8> beddata;
    QVector<Transponder> transponders = readTransponders();
    QTextStream cout(stdout, QIODevice::WriteOnly);
    if(transponders.size()==1) {
        NDEFMessage message = _NFCManager->readNDEFMessage(&transponders[0]);
        if(transponders[0].getType() == DESFIRE_MIFARE) {
            QVector<NDEFRecord*> records = QVector<NDEFRecord*>::fromStdVector(message.getNdefRecords());
            if(records.size()>=8) {
                cout<<"Daten: "<<QString::fromStdString(records[7]->getPayload()).toAscii()<<endl<<flush;
                QByteArray data = QByteArray::fromHex(QString::fromStdString(records[7]->getPayload()).toAscii());
                dec(cout)<<"also: "<<(quint8) data[0]<<" "<<(quint8) data[1]<<endl;
                beddata.push_back((quint8) data[0]);
                beddata.push_back((quint8) data[1]);
            } else {
                cout<<"Dieser Transponder enthält keine oder zu wenig NDEF-Records!"<<endl;
                cout<<"Habe "<<records.size()<<" Records gefunden."<<endl;
            }
        } else {
            cout<<"Dieser Transponder ist kein DESFIRE-Transponder!"<<endl;
        }
    } else {
        cout<<"Bitte nur einen Transponder auf das Bett legen!"<<endl;
    }
    return beddata;
}
