#!/usr/bin/env python
# -*- coding: utf-8 -*-

from flask import render_template, redirect, abort, json, request, Response
from flask.views import MethodView

from server import db
from model import Person, Befund, Allergie, allergien

import datetime
from dateutil import parser

class PersonController(MethodView):

    def get(self, kknummer):
        f = request.args.get('f', 'json')
        if kknummer is None:
            if f == "html":
                return render_template("person/get_all.html", personen=Person.query.all())
            if f == "json":
                personen = []
                for person in Person.query.all():
                    personen.append({
                        "vorname":       person.vorname,
                        "nachname":      person.nachname,
                        "geschlecht":    person.geschlecht,
                        "geburtstag":    str(person.geburtstag),
                        "notfallnummer": person.notfallnummer,
                        "adresse":       person.adresse,
                        "plz":           person.plz,
                        "ort":           person.ort,
                        "kknummer":      person.kknummer,
                        "personal":      person.personal,
                        "passwort":      person.passwort,
                        "blutgruppe":    person.blutgruppe,
                        "befunde":       [b.id for b in person.befunde],
                        "allergien":     [a.name for a in person.allergien]
                    })
                return Response(json.dumps(personen), mimetype='application/json')
            else:
                abort(404)
        else:
            if f == "json":
                person = Person.query.get(kknummer)
                if not person:
                    abort(404)
                return Response(json.dumps({
                        "vorname":       person.vorname,
                        "nachname":      person.nachname,
                        "geschlecht":    person.geschlecht,
                        "geburtstag":    str(person.geburtstag),
                        "notfallnummer": person.notfallnummer,
                        "adresse":       person.adresse,
                        "plz":           person.plz,
                        "ort":           person.ort,
                        "kknummer":      person.kknummer,
                        "personal":      person.personal,
                        "passwort":      person.passwort,
                        "blutgruppe":    person.blutgruppe,
                        "befunde":       [b.id for b in person.befunde],
                        "allergien":     [a.name for a in person.allergien]
                    }), mimetype='application/json')
            else:
                abort(404)

    def post(self):
        p = Person()
        p.kknummer = request.form.get("kknummer")
        if p.kknummer == None or not Person.query.get(p.kknummer) == None:
            abort(400)
        p.vorname = request.form.get("vorname")
        p.nachname = request.form.get("nachname")
        p.geschlecht = request.form.get("geschlecht")
        if not request.form.get("geburtstag") == None:
            p.geburtstag = parser.parse(request.form.get("geburtstag"))
        p.notfallnummer = request.form.get("notfallnummer")
        p.adresse = request.form.get("adresse")
        p.plz = request.form.get("plz")
        p.ort = request.form.get("ort")
        p.passwort = request.form.get("passwort")
        p.personal = request.form.get("personal")
        p.blutgruppe = request.form.get("blutgruppe")
        p.allergien = []
        a_list = request.form.get("allergien")
        if a_list:
            for a_elem in a_list.split(","):
                a = Allergie.query.get(a_elem.strip())
                if a == None:
                    a = Allergie(a_elem.strip())
                    db.session.add(a)
                p.allergien.append(a)
        db.session.add(p)
        db.session.commit()
        return redirect("/personen")

    def delete(self, kknummer):
        p = Person.query.get(kknummer)
        if p == None:
            abort(404)
        for b in p.befunde:
            db.session.delete(b)
        db.session.delete(p)
        db.session.commit()
        return redirect("/personen")

    def put(self, kknummer):
        p = Person.query.get(kknummer)
        if p == None:
            abort(404)
        if request.form.get("vorname"):
            p.vorname = request.form.get("vorname")
        if request.form.get("nachname"):
            p.nachname = request.form.get("nachname")
        if request.form.get("geschlecht"):
            p.geschlecht = request.form.get("geschlecht")
        if request.form.get("geburtstag"):
            p.geburtstag = parser.parse(request.form.get("geburtstag"))
        if request.form.get("notfallnummer"):
            p.notfallnummer = request.form.get("notfallnummer")
        if request.form.get("adresse"):
            p.adresse = request.form.get("adresse")
        if request.form.get("plz"):
            p.plz = request.form.get("plz")
        if request.form.get("ort"):
            p.ort = request.form.get("ort")
        if request.form.get("passwort"):
            p.passwort = request.form.get("passwort")
        if request.form.get("personal"):
            p.personal = request.form.get("personal")
        if request.form.get("blutgruppe"):
            p.blutgruppe = request.form.get("blutgruppe")
        if request.form.get("allergien"):
            p.allergien = []
            a_list = request.form.get("allergien")
            for a_elem in a_list.split(","):
                a = Allergie.query.get(a_elem.strip())
                if a == None:
                    a = Allergie(a_elem.strip())
                    db.session.add(a)
                p.allergien.append(a)
        db.session.commit()
        return redirect("/personen")

class BefundController(MethodView):

    def get(self, befund_id):
        f = request.args.get('f', 'json')
        if befund_id is None:
            if request.args.get("person"):
                bs = Befund.query.filter(Befund.person_kknummer == request.args.get("person"))
            else:
                bs = Befund.query.all()
            if f == "html":
                return render_template("befund/get_all.html", befunde=bs)
            if f == "json":
                befunde = []
                for befund in bs:
                    befunde.append({
                        "id":        befund.id,
                        "titel":     befund.titel,
                        "bild":      befund.bild,
                        "datum":     str(befund.datum),
                        "text":      befund.text,
                        "person":    befund.person.kknummer
                    })
                return Response(json.dumps(befunde), mimetype='application/json')
            else:
                abort(404)
        else:
            if f == "json":
                befund = Befund.query.get(befund_id)
                if not befund:
                    abort(404)
                return Response(json.dumps({
                        "id":        befund.id,
                        "titel":     befund.titel,
                        "bild":      befund.bild,
                        "datum":     str(befund.datum),
                        "text":      befund.text,
                        "person":    befund.person.kknummer
                    }), mimetype='application/json')
            else:
                abort(404)

    def post(self):
        b = Befund()
        kknummer = request.form.get("person")
        if kknummer == None or Person.query.get(kknummer) == None:
            abort(400)
        b.titel = request.form.get("titel")
        if not request.form.get("datum") == None:
            b.datum = parser.parse(request.form.get("datum"))
        b.bild = request.form.get("bild")
        b.text = request.form.get("text")
        b.person_kknummer = kknummer
        db.session.add(b)
        db.session.commit()
        return redirect("/befunde")

    def delete(self, befund_id):
        b = Befund.query.get(befund_id)
        if b == None:
            abort(404)
        db.session.delete(b)
        db.session.commit()
        return redirect("/befunde")

    def put(self, befund_id):
        b = Befund.query.get(befund_id)
        if b == None:
            abort(404)
        kknummer = request.form.get("person")
        if kknummer and Person.query.get(kknummer) == None:
            abort(400)
        elif kknummer:
            b.person_kknummer = kknummer
        if request.form.get("titel"):
            b.titel = request.form.get("titel")
        if request.form.get("datum"):
            b.datum = parser.parse(request.form.get("datum"))
        if request.form.get("bild"):
            b.bild = request.form.get("bild")
        if request.form.get("text"):
            b.text = request.form.get("text")
        db.session.commit()
        return redirect("/befunde")

class AllergieController(MethodView):

    def get(self, name):
        f = request.args.get('f', 'json')
        if name is None:
            if f == "html":
                return render_template("allergie/get_all.html", allergien=Allergie.query.all())
            if f == "json":
                allergien = []
                for allergie in Allergie.query.all():
                    allergien.append({
                        "name":     allergie.name
                    })
                return Response(json.dumps(allergien), mimetype='application/json')
            else:
                abort(404)
        else:
            if f == "json":
                allergie = Allergie.query.get(name)
                if not allergie:
                    abort(404)
                return Response(json.dumps({
                        "name":     allergie.name
                    }), mimetype='application/json')
            else:
                abort(404)

    def post(self):
        if request.form.get("name") == None or not Allergie.query.get(request.form.get("name")) == None:
            abort(400)
        a = Allergie(request.form.get("name"))
        db.session.add(a)
        db.session.commit()
        return redirect("/allergien")

    def delete(self, name):
        a = Allergie.query.get(name)
        if a == None:
            abort(404)
        db.session.delete(a)
        db.session.commit()
        return redirect("/allergien")

    def put(self, name):
        abort(405)
