//Copyright 2011 INKA - HTW Berlin
//
//Licensed under the Apache License, Version 2.0 (the "License");
//you may not use this file except in compliance with the License.
//You may obtain a copy of the License at
//
//http://www.apache.org/licenses/LICENSE-2.0
//
//Unless required by applicable law or agreed to in writing, software
//distributed under the License is distributed on an "AS IS" BASIS,
//WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//See the License for the specific language governing permissions and
//limitations under the License.
//
//  Created by Maurus Rohrer maurus.rohrer@gmail.com
//

#include "LibnfcDeviceManager.h"
#include "DesfireTransponder.h"
#include "UltralightTransponder.h"
#include "ClassicTransponder.h"
#include "MifareDesfire.h"

#include <math.h>

bool LibnfcDeviceManager::connect(){
    device = nfc_connect(NULL);
    if (device == NULL) {
        //connect failed
        return false;
    }else{
        //configure device
        // Set connected NFC device to initiator mode
        nfc_initiator_init (device);
        nfc_configure (device, NDO_ACTIVATE_FIELD, true);
        return true;
    }
}

bool LibnfcDeviceManager::disconnect(){
    nfc_disconnect(device);
    return true;  
}


std::vector<Transponder>  LibnfcDeviceManager::readTransponders(){
        std::vector<Transponder> transponders;
        tag = freefare_get_tags (device);
        if(tag != NULL) {
            while (*tag) {
                switch (freefare_get_tag_type (*tag)) {
                    case DESFIRE:
                    {
                        transponders.push_back(mifare_desfire::initDesfireTransponder(*tag));
                        break;
                    }
                    case ULTRALIGHT:
                    {
                        UltralightTransponder ut(freefare_get_tag_uid (*tag));
                        transponders.push_back(ut);
                        break;
                    }
                    case CLASSIC_1K:
                    {
                        ClassicTransponder ct1(CLASSIC_1K_MIFARE, freefare_get_tag_uid (*tag));
                        transponders.push_back(ct1);
                        break; 
                    }

                    case CLASSIC_4K:
                    {
                        ClassicTransponder ct4(CLASSIC_4K_MIFARE, freefare_get_tag_uid (*tag));
                        transponders.push_back(ct4);
                        break;
                    }

                    default:
                        break;
                }
            tag++;
            }
    }
    return transponders;
}


bool LibnfcDeviceManager::isNDEFConform(Transponder* transponder){
    //scan all tag on reader again!
    tag = freefare_get_tags (device);
    if(tag != NULL) {
        while (*tag) {
            //check if transponder is the one on reader
            if (freefare_get_tag_uid (*tag) == transponder->getID()) {
                //switch to transponder type 
                switch (freefare_get_tag_type (*tag)) {
                    case DESFIRE:
                    {
                        return mifare_desfire::isNDEFConform(*tag);
                        break;
                    }
                    case ULTRALIGHT:
                    {
                        break;
                    }
                    case CLASSIC_1K:
                    {
                        break; 
                    }
                        
                    case CLASSIC_4K:
                    {
                        break;
                    }
                        
                    default:
                        break;
                }
            }
            tag++;
        }
        //transponder not found on reader
    }
    return false;
}

bool LibnfcDeviceManager::makeNDEFConform(Transponder* transponder){
    //scan all tag on reader again!
    tag = freefare_get_tags (device);
    if(tag != NULL) {
        while (*tag) {
            //check if transponder is the one on reader
            if (freefare_get_tag_uid (*tag) == transponder->getID()) {
                //switch to transponder type 
                switch (freefare_get_tag_type (*tag)) {
                    case DESFIRE:
                    {
                        return mifare_desfire::makeNDEFConform(*tag);
                        break;
                    }
                    case ULTRALIGHT:
                    {
                        break;
                    }
                    case CLASSIC_1K:
                    {
                        break; 
                    }
                        
                    case CLASSIC_4K:
                    {
                        break;
                    }
                        
                    default:
                        break;
                }
            }
            tag++;
        }
        //transponder not found on reader
    }
    return false;

}

bool LibnfcDeviceManager::writeNDEFMessage(Transponder* transponder,NDEFMessage* message){
    //scan all tag on reader again!
    tag = freefare_get_tags (device);
    if(tag != NULL) {
        while (*tag) {
            //check if transponder is the one on reader
            if (freefare_get_tag_uid (*tag) == transponder->getID()) {
                
                switch (freefare_get_tag_type (*tag)) {
                    case DESFIRE:
                    {
                        return mifare_desfire::writeNDEFMessage(*tag, message);
                        break;
                    }
                    case ULTRALIGHT:
                    {
                        break;
                    }
                    case CLASSIC_1K:
                    {
                        break; 
                    }
                        
                    case CLASSIC_4K:
                    {
                        break;
                    }
                        
                    default:
                        break;
                }

            }
            tag++;
        }
    }
    return false;
    
}

NDEFMessage LibnfcDeviceManager::readNDEFMessage(Transponder* transponder){
    //scan all tag on reader again!
    tag = freefare_get_tags (device);
    if(tag != NULL) {
        while (*tag) {
            //check if transponder is the one on reader
            if (freefare_get_tag_uid (*tag) == transponder->getID()) {
                
                switch (freefare_get_tag_type (*tag)) {
                    case DESFIRE:
                    {
                        return mifare_desfire::readNDEFMessage(*tag);
                    }
                    case ULTRALIGHT:
                    {
                        break;
                    }
                    case CLASSIC_1K:
                    {
                        break; 
                    }
                        
                    case CLASSIC_4K:
                    {
                        break;
                    }
                        
                    default:
                        break;
                }
                
            }
            tag++;
        }
    }
    return NULL;
}





