//Copyright 2011 INKA - HTW Berlin
//
//Licensed under the Apache License, Version 2.0 (the "License");
//you may not use this file except in compliance with the License.
//You may obtain a copy of the License at
//
//http://www.apache.org/licenses/LICENSE-2.0
//
//Unless required by applicable law or agreed to in writing, software
//distributed under the License is distributed on an "AS IS" BASIS,
//WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//See the License for the specific language governing permissions and
//limitations under the License.
//
//  Created by Maurus Rohrer maurus.rohrer@gmail.com


#ifndef MetraTecDeviceManager_
#define MetraTecDeviceManager_


#include <string>
#include <iostream>
#include <vector>
#include <sstream>
#include <iomanip> 
#include "RFDeviceManager.h"
#include "Transponder.h"
#include "ICodeTransponder.h"

struct is_id {
    bool operator ()(const std::string &str) {
        return !str.empty() && str.length() != 16;
    }
};

struct is_data {
    bool operator ()(const std::string &str) {
        return !str.empty() && str.compare(0,2,"00");
    }
};

class MetraTecDeviceManager : public RFDeviceManager
{
public:
    MetraTecDeviceManager(std::string);
    bool connect();
    bool disconnect();
    std::vector<Transponder> readTransponders();
    bool isNDEFConform(Transponder*);
    bool makeNDEFConform(Transponder*);
    bool writeNDEFMessage(Transponder*,NDEFMessage*);
    NDEFMessage readNDEFMessage(Transponder*);
    bool configureReader(TransponderType);
    std::vector<std::string> writeMetraTecCommand(std::string);
    
protected:
    std::string device;
    int fileDescriptor;
    TransponderType active_type;
private:
    void splitString(std::string, std::vector<std::string>*);
    std::string byteToHexString(unsigned char);
//    std::string charStringToHexString(std::string);
//    std::string hexStringToCharString(std::string);
    bool writeDataOnBlockOnTranspond(std::string, std::string, int);
    std::string readDataOnBlockOnTranspond(std::string, int);
    
};

#endif
