//Copyright 2011 INKA - HTW Berlin
//
//Licensed under the Apache License, Version 2.0 (the "License");
//you may not use this file except in compliance with the License.
//You may obtain a copy of the License at
//
//http://www.apache.org/licenses/LICENSE-2.0
//
//Unless required by applicable law or agreed to in writing, software
//distributed under the License is distributed on an "AS IS" BASIS,
//WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//See the License for the specific language governing permissions and
//limitations under the License.
//
//  Created by Maurus Rohrer maurus.rohrer@gmail.com
//



#include "MifareDesfire.h"
#include "DesfireTransponder.h"

#include <math.h>
#include <string.h>


namespace mifare_desfire
{
    Transponder initDesfireTransponder(MifareTag tag){
        DesfireTransponder dt(freefare_get_tag_uid (tag), tag);
        //get and clac size
        //MifareTag testing = dt.getMifareTag();
        struct mifare_desfire_version_info info;
        mifare_desfire_connect (tag);
        mifare_desfire_get_version (tag, &info);
        dt.setSize((int)pow(2, info.hardware.storage_size>>1));
        mifare_desfire_disconnect (tag);
        return dt;
    }
    
    
    bool isNDEFConform(MifareTag tag){
        //connect to tag
        
        if(mifare_desfire::connectCCApplication(tag)){
            char buffer[2048];
            int res = mifare_desfire_read_data(tag, mifare_desfire::ccFileNo, 0, 0, buffer);
            if(res < 0){
                std::cout << "read on application failed" << std::endl;
                mifare_desfire_disconnect(tag);
                return false;
            }
            //TODO: seam that strcmp is not working!!!!!
            if(strcmp(buffer, mifare_desfire::capability_container_file_content )==0){
                mifare_desfire_disconnect (tag);
                return true;
            }

        }
        return false;
    }
    
    
    
    bool makeNDEFConform(MifareTag tag){
        //connect to card
        if (connectTag(tag)) {
            //format card
            int res = mifare_desfire_format_picc (tag);
            if(res<0){
                std::cout << "format card failed" << std::endl;
                mifare_desfire_disconnect (tag);
                return false;
            }
            //create application
            MifareDESFireAID aid = mifare_desfire_aid_new(0xEEEE10);
            res = mifare_desfire_create_application (tag, aid, 0x09, 1);
            if (res < 0){
                std::cout << "application creation failed" << std::endl; 
                mifare_desfire_disconnect (tag);
                return false;
            }
            if (!connectCCApplication(tag)) {
                std::cout << "connect failed" << std::endl; 
                mifare_desfire_disconnect (tag);
                return false;
            }
            res = mifare_desfire_create_std_data_file(tag,0x03,0x00,0xE000,0x00000F);
            if(res < 0){
                std::cout << "creat cc on application failed" << std::endl; 
                mifare_desfire_disconnect (tag);
                return false;
            }
            res = mifare_desfire_write_data(tag,0x03,0,sizeof(capability_container_file_content),capability_container_file_content);
            if (res < 0){
                std::cout << "write cc content failed" << std::endl; 
                mifare_desfire_disconnect (tag);
                return false;   
            }  
            res = mifare_desfire_create_std_data_file(tag,0x04,0x00,0xEEE0,0x000DAC);
            if (res < 0){
                std::cout << "creat ndef file on application failed" << std::endl; 
                mifare_desfire_disconnect (tag);
                return false; 
            }
            mifare_desfire_disconnect (tag);
            return true; 

        }
        
        return false;
    }
    
    bool writeNDEFMessage(MifareTag tag,NDEFMessage* message){
        if (mifare_desfire::connectCCApplication(tag)) {
            std::string ndefMes = mifare_desfire::intTo4HexString(message->getNDEFMessageLength());
          //  ndefMes.append(message->getNDEFMessageLengthHex());
            ndefMes.append(message->getNDEFMessage());
            
            std::string messageToWrite = mifare_desfire::hexStringToCharString(ndefMes);

            //TODO; check ndef size if it fits on Transponder
            
            int res = mifare_desfire_write_data (tag, ndefDataFileNo, 0, messageToWrite.length(), (char *)messageToWrite.c_str());
            //int res = mifare_desfire_write_data (tag[0], 0x04, 0, 30, (uint8_t *)"Some data to write to the card");
            if (res<=0) {
                std::cout << "write on application failed" << std::endl;
                mifare_desfire_disconnect (tag);
                return false;
            }
            mifare_desfire_disconnect (tag);
            return true;
        }
        return false;
    }
    
    
    NDEFMessage readNDEFMessage(MifareTag tag){
         if (mifare_desfire::connectCCApplication(tag)) {
             char ndefBuffer[4096];
             int res = mifare_desfire_read_data(tag, ndefDataFileNo, 0, 0, ndefBuffer);
             if(res < 0){
                 std::cout << "read on application failed" << std::endl;
                 mifare_desfire_disconnect(tag);
                 return NDEFMessage();
             }
             
             std::string ndefMessageHex =charArrayToHexString(ndefBuffer,res);
             int size = hexStringToInt(ndefMessageHex.substr(0,4));
             if (size==0) {
                 return NDEFMessage();
             }
             NDEFMessage ndefMessage = NDEFMessage::parseNDEFMessage(ndefMessageHex.substr(4,size));
             return ndefMessage;
         }
        
        return NDEFMessage();
    }
                
            
    
    bool connectCCApplication(MifareTag tag){
        if (connectTag(tag)) {
            MifareDESFireAID *aids=NULL;
            size_t count;
            mifare_desfire_get_application_ids (tag, &aids, &count);
            if (count==0) return false;
            for(int i=0; i<count; i++){
                uint32_t aidNummber = mifare_desfire_aid_get_aid(aids[i]);
                if(aidNummber == mifare_desfire::ndefAid){
                    int res = mifare_desfire_select_application (tag, aids[i]);
                    if(res < 0){
                        std::cout << "select application failed" << std::endl;
                        mifare_desfire_disconnect(tag);
                        return false;
                    }
                    MifareDESFireKey key = mifare_desfire_des_key_new_with_version (mifare_desfire::standard_ndef_key);
                    res = mifare_desfire_authenticate (tag, 0, key);
                    if(res < 0){
                        std::cout << "auth on application failed" << std::endl;
                        mifare_desfire_disconnect(tag);
                        return false;
                    }
                    return true;
                }
            }
            mifare_desfire_disconnect(tag);
            return false;

        }            
        mifare_desfire_disconnect(tag);
        return false;
    }

    bool connectTag(MifareTag tag){
        int res = mifare_desfire_connect (tag);
        if(res < 0){
            std::cout << "connect failed" << std::endl;
        }
        
        //authenticate with standart key
        MifareDESFireKey key = mifare_desfire_des_key_new_with_version (mifare_desfire::standard_ndef_key);
        res = mifare_desfire_authenticate (tag, 0, key);
        if(res < 0){
            std::cout << "auth on card failed" << std::endl;
            mifare_desfire_disconnect(tag);
            return false;
        }
        return true;
    }
    
    std::string hexStringToCharString(std::string str){
        std::ostringstream outputStream;
        for(size_t i = 0; i + 1 <= str.size(); i += 2){
            unsigned short s;
            std::istringstream(str.substr(i, 2)) >> std::hex >> s;
            outputStream << static_cast<unsigned char>(s);
        }
        return outputStream.str();
    }
    
    std::string intTo4HexString(int c){
        std::ostringstream out;
        out << std::hex << std::setw(4) << std::setfill('0') << c;
        return out.str();
    }
    
    
    std::string charStringToHexString(std::string str) {
        std::string returnValue;
        for(std::string::const_iterator i = str.begin(); i != str.end(); ++i)
        {
            returnValue.append(byteToHexString(*i));
        }
        return returnValue;
    }
    
    std::string charArrayToHexString(char* str, int length) {
        std::string returnValue;
        for(int i = 0; i <length; ++i)
        {
            returnValue.append(byteToHexString(str[i]));
        }
        return returnValue;
    }
    
    std::string byteToHexString(unsigned char c){
        std::ostringstream out;
        out << std::hex << std::setw(2) << std::setfill('0') << static_cast<unsigned int>(static_cast<unsigned char>(c));
        return out.str();
    }
    
    int hexStringToInt(std::string str){
        unsigned int x;   
        std::stringstream ss;
        ss << std::hex << str;
        ss >> x;
        return x;
    }

    
    

}
