//Copyright 2011 INKA - HTW Berlin
//
//Licensed under the Apache License, Version 2.0 (the "License");
//you may not use this file except in compliance with the License.
//You may obtain a copy of the License at
//
//http://www.apache.org/licenses/LICENSE-2.0
//
//Unless required by applicable law or agreed to in writing, software
//distributed under the License is distributed on an "AS IS" BASIS,
//WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//See the License for the specific language governing permissions and
//limitations under the License.
//
//  Created by Maurus Rohrer maurus.rohrer@gmail.com
//

#ifndef MifareDesfire_
#define MifareDesfire_

#include <nfc/nfc.h>
#include <freefare.h>
#include "Transponder.h"
#include "NDEFMessage.h"

namespace mifare_desfire {
    Transponder initDesfireTransponder(MifareTag);
    bool connectCCApplication(MifareTag);
    bool connectTag(MifareTag);
    bool isNDEFConform(MifareTag);
    bool makeNDEFConform(MifareTag);
    bool writeNDEFMessage(MifareTag,NDEFMessage*);
    NDEFMessage readNDEFMessage(MifareTag);
    
    //helper 
    //TODO: make helper namespace and put code there
    std::string hexStringToCharString(std::string str);
    std::string intTo4HexString(int);
    std::string byteToHexString(unsigned char);
    std::string charStringToHexString(std::string str);
    std::string charArrayToHexString(char*,int);
    int hexStringToInt(std::string);
    
    //standard values
    static uint8_t standard_ndef_key[8]  = { 0x00
        , 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
    static uint32_t ndefAid = 0xEEEE10;
    static char capability_container_file_content[15] = {
        0x00, 0x0F,                 // CCLEN: Size of this capability container.CCLEN values are between 000Fh and FFFEh
        0x10,                       // Mapping version
        0x00, 0x3B,                 // MLe: Maximum data size that can be read using a single ReadBinary command. MLe = 000Fh-FFFFh
        0x00, 0x34,                 // MLc: Maximum data size that can be sent using a single UpdateBinary command. MLc = 0001h-FFFFh
        0x04, 0x06, 0xE1, 0x04,     // TLV
        0x0d, 0xac,                 // NDEF File size
        0x00,                       // free read access
        0x00                        // free write acces
    };
    static uint8_t ccFileNo = 0x03;
    static uint8_t ndefDataFileNo = 0x04;
}

#endif //MifareDesfire_