//Copyright 2011 INKA - HTW Berlin
//
//Licensed under the Apache License, Version 2.0 (the "License");
//you may not use this file except in compliance with the License.
//You may obtain a copy of the License at
//
//http://www.apache.org/licenses/LICENSE-2.0
//
//Unless required by applicable law or agreed to in writing, software
//distributed under the License is distributed on an "AS IS" BASIS,
//WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//See the License for the specific language governing permissions and
//limitations under the License.
//
//  Created by Maurus Rohrer maurus.rohrer@gmail.com

#include "NDEFMessage.h"


#include <iostream>

#include "MifareDesfire.h"

NDEFMessage::NDEFMessage(){
    //standard constructor
}

NDEFMessage::NDEFMessage(NDEFRecord* record){
    ndefRecords.push_back(record);
}


bool NDEFMessage::addNdefRecord(NDEFRecord* rec){
    ndefRecords.push_back(rec);
    return true;
}

std::string NDEFMessage::getTLV(){
    //create TLV
    
    // Type always "03h" NDEF Type
    std::string tlv = "03";
    //wirte empty NDEFMessage
    if(ndefRecords.empty()) {
        tlv.append("00"); //empty NDEF Message
        tlv.append("FE"); //TLV terminator
        return tlv;
    }
    //get NDEFMessage (Header and Payload)
    std::string  v = getNDEFMessage();
    //get lenht of value
    std::string l = byteToHexString(v.length());
    tlv.append(l);
    // add value

    tlv.append(v);
    //add tlv terminator
    tlv.append("FE");
    return tlv;
}

std::string NDEFMessage::getNDEFMessage(){
    std::string v;
    //create messages and headers 
    //TODO: Chunked Payload not implemented
    //      ID field not implemented
    //      only support TNT: NFC Forum well-know type
    
    for(int pos = 0; pos < ndefRecords.size(); pos++){
        int headerByte = 0x00;
        if(ndefRecords.size() == 1){
            //is first an last record in message
            headerByte |= 1 << 7; //Messege Begin 
            headerByte |= 1 << 6; //Message End
        } else {
            //has more than one record
            //check if first record
            if(pos==0){
                headerByte |= 1 << 7; //Messege Begin 
            }else if(pos==(ndefRecords.size()-1)){
                headerByte |= 1 << 6; //Messege End 
            }
        }
        if (ndefRecords[pos]->getPayload().length()<200) {
            headerByte |= 1 << 4; //Short Record
        }else{
            headerByte |= 0 << 4; //Long Record
        }
        //Type 
        headerByte |= 1 << 0; //TNT : Well know NFC Type
        v.append(byteToHexString(headerByte));
        
        //get NDEFRecord Header
        v.append(ndefRecords[pos]->getNDEFRecordHeader());
        //get NDEFRecordPayload in HEX rep.
        v.append(ndefRecords[pos]->getPayloadHex());
    }
    return v;
}

NDEFMessage NDEFMessage::parseNDEFMessage(std::string message){
    NDEFMessage ndefMessage;
    int recordSize =0;
    bool endOfMessage=false;
    //std::string tmp = message;
    while (!endOfMessage && !message.empty()) {
        //TODO: analyse header byte to get information
        int headerByte = mifare_desfire::hexStringToInt(message.substr(0,2));
        if(headerByte & (1 << 6)){
            endOfMessage=true;
        }
        if(headerByte & (1 << 0)){
            //TNT: Well know NFC Type (parse message)
            bool shortRecord = false;
            if(headerByte & (1 << 4)){
                shortRecord=true;
            }else{
                shortRecord=false;
            }
            NDEFRecord* record = NDEFRecord::parseNdefRecord(message, shortRecord);
            if (record) {
                ndefMessage.addNdefRecord(record);
            }else{
                std::cout << "error parsing record! exit parsing" << std::endl;
                return ndefMessage;
            }
            //get size of TNT record (first to bytes are sizes 1. type size 2. payload size) plus 2 bytes (1. type length 2. payload length
            if (shortRecord) {
                
                //get size of TNT record (first to bytes are sizes 1. type size 2. payload size) plus 2 bytes (1. type length 2. payload length
                recordSize = mifare_desfire::hexStringToInt(message.substr(2,2)) + mifare_desfire::hexStringToInt(message.substr(4,2)) + 3;
            }else{
                recordSize =mifare_desfire::hexStringToInt(message.substr(2,2)) + 6;
                int test = mifare_desfire::hexStringToInt(message.substr(4,8));
                recordSize+= test;
            }
            // message lenght in hex (times 2)
            recordSize = recordSize*2;
            message = message.substr(recordSize, message.length()-recordSize);

        }else{
            //TODO parse other types than Well know NFC Type
            return ndefMessage;
        }
    }
    
    
    return ndefMessage;
}

std::string NDEFMessage::getNDEFMessageLengthHex(){
    std::cout << "ha hier ist die lengemann" << std::endl;
    printf("ha ha %lu", getNDEFMessage().length());
    std::string l = byteToHexString(getNDEFMessage().length());
    std::cout << "ha hier ist die lengemann nach hex" << l << std::endl;
    return l;
}

int NDEFMessage::getNDEFMessageLength(){
    return getNDEFMessage().length();
}




std::vector<NDEFRecord*> NDEFMessage::getNdefRecords(){
    return ndefRecords;
    
}

std::string NDEFMessage::byteToHexString(unsigned char c){
    std::ostringstream out;
    out << std::hex << std::setw(2) << std::setfill('0') << static_cast<unsigned int>(static_cast<unsigned char>(c));
    return out.str();
}




std::string NDEFMessage::charStringToHexString(std::string str) {
    std::string returnValue;
    for(std::string::const_iterator i = str.begin(); i != str.end(); ++i)
    {
        returnValue.append(byteToHexString(*i));
    }
    return returnValue;
}


std::string NDEFMessage::hexStringToCharString(std::string str){
    std::ostringstream outputStream;
    for(size_t i = 0; i + 1 <= str.size(); i += 2){
        unsigned short s;
        std::istringstream(str.substr(i, 2)) >> std::hex >> s;
        outputStream << static_cast<unsigned char>(s);
    }
    return outputStream.str();
}