//Copyright 2011 INKA - HTW Berlin
//
//Licensed under the Apache License, Version 2.0 (the "License");
//you may not use this file except in compliance with the License.
//You may obtain a copy of the License at
//
//http://www.apache.org/licenses/LICENSE-2.0
//
//Unless required by applicable law or agreed to in writing, software
//distributed under the License is distributed on an "AS IS" BASIS,
//WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//See the License for the specific language governing permissions and
//limitations under the License.
//
//  Created by Maurus Rohrer maurus.rohrer@gmail.com

#include "NDEFRecord.h"
#include "UriNDEFRecord.h"
#include "TextNDEFRecord.h"

#include "MifareDesfire.h"

NDEFRecord::NDEFRecord(NDEFRecordType type, std::string payload): type(type), payload(payload) {
}


void NDEFRecord::setPayload(std::string pay){
    payload = pay;
}

std::string NDEFRecord::byteToHexString(unsigned char c){
    std::ostringstream out;
    out << std::hex << std::setw(2) << std::setfill('0') << static_cast<unsigned int>(static_cast<unsigned char>(c));
    return out.str();
}


std::string NDEFRecord::charStringToHexString(std::string* str) {
    std::string returnValue;
    for(std::string::const_iterator i = str->begin(); i != str->end(); ++i)
    {
        returnValue.append(byteToHexString(*i));
    }
    return returnValue;
}

NDEFRecord* NDEFRecord::parseNdefRecord(std::string record, bool isShort){
    NDEFRecord* ndefRecord;
    std::string test =record;
    if (record.substr(2,2)=="01") {
        //set Type Postion
        int typePos=0;
        if(isShort){
            typePos=6; 
        }else{
            typePos=12;
        }
        //Type Length is 1
        if (record.substr(typePos,2)=="54") {
            //record is text record
            ndefRecord = TextNDEFRecord::parseTextNDEFRecord(record,isShort);
        }else if (record.substr(typePos,2)=="55"){
            //record is uri record
            ndefRecord = UriNDEFRecord::parseUriNDEFRecord(record,isShort);
        }else{
            //unknown record return null
            return NULL;
        }
    }
    return ndefRecord;
}