//Copyright 2011 INKA - HTW Berlin
//
//Licensed under the Apache License, Version 2.0 (the "License");
//you may not use this file except in compliance with the License.
//You may obtain a copy of the License at
//
//http://www.apache.org/licenses/LICENSE-2.0
//
//Unless required by applicable law or agreed to in writing, software
//distributed under the License is distributed on an "AS IS" BASIS,
//WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//See the License for the specific language governing permissions and
//limitations under the License.
//
//  Created by Maurus Rohrer maurus.rohrer@gmail.com

#include "TextNDEFRecord.h"

#include "MifareDesfire.h"



TextNDEFRecord::TextNDEFRecord(TextEnc enc, std::string lang, std::string payl): NDEFRecord(TEXT, payl), encoding(enc), language(lang){
    
};


TextEnc TextNDEFRecord::getEncoding(){
    return encoding;
}

std::string TextNDEFRecord::getLanguage(){
    return language;
}


std::string TextNDEFRecord::getPayload(){
    return payload;
}

std::string TextNDEFRecord::getPayloadHex(){
    return charStringToHexString(&payload);
}

NDEFRecordType TextNDEFRecord::getNDEFRecordType(){
    return type;
}

std::string TextNDEFRecord::getNDEFRecordHeader(){
    std::string header;
    header.append("01"); //Type Length URI is 1
    if (payload.length()<200) {
        //add payloadlength Short Record (8 Bit)
        header.append(byteToHexString(payload.length()+language.length()+1)); 
    }else{
        //Long Record 
        //first to byte 00
        header.append("0000");
        header.append(mifare_desfire::intTo4HexString((int)payload.length()+(int)language.length()+1));
    }
    //ID length ignored
    header.append("54"); // Type "T" 0x54h
    
    //set status byte
    int statusByte = (int)language.size(); //bit 0-5 size of language
    if (encoding== UTF8) {
        statusByte |= 0 << 7; //bit 7, 0b if UTF-8
    }else{
        statusByte |= 1 << 7; //bit 7, 1b if UTF16
    }
    statusByte |= 0 << 6;                //bit 6 always 0b
    header.append(byteToHexString(statusByte));
    
    //apend language
    header.append(charStringToHexString(&language));
    return header;
}

TextNDEFRecord* TextNDEFRecord::parseTextNDEFRecord(std::string textrecord, bool isShort){
    //get payload length and language length 
    
    int size=0;
    int readPos=0;
    if(isShort){
        size = mifare_desfire::hexStringToInt(textrecord.substr(4,2));
        readPos=8;
    }else{
        size = mifare_desfire::hexStringToInt(textrecord.substr(4,8));
        readPos=14;
    }
    // minus -1 for the status byte and trough 2 data still in hex
    size = (size -1)*2;
    //get status byte
    int statusByte = mifare_desfire::hexStringToInt(textrecord.substr(readPos,2));
    TextEnc enc;
    if (statusByte & (1 << 7)) {
        enc = UTF16;
        //clear bit to get lenght
        statusByte &= ~(1 << 7);
    }else{
        enc = UTF8;
    }
    int languageSize = (int)statusByte*2;
    readPos+=2;
    std::string language = mifare_desfire::hexStringToCharString(textrecord.substr(readPos,languageSize));
    std::string payload = mifare_desfire::hexStringToCharString(textrecord.substr(readPos+languageSize, size-languageSize));
    return new TextNDEFRecord(enc, language, payload);
       

}