//Copyright 2011 INKA - HTW Berlin
//
//Licensed under the Apache License, Version 2.0 (the "License");
//you may not use this file except in compliance with the License.
//You may obtain a copy of the License at
//
//http://www.apache.org/licenses/LICENSE-2.0
//
//Unless required by applicable law or agreed to in writing, software
//distributed under the License is distributed on an "AS IS" BASIS,
//WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//See the License for the specific language governing permissions and
//limitations under the License.
//
//  Created by Maurus Rohrer maurus.rohrer@gmail.com

#include "UriNDEFRecord.h"

#include <iostream>
#include "MifareDesfire.h"

UriNDEFRecord::UriNDEFRecord(UriType type, std::string payload): NDEFRecord(URI, payload), uriType(type){
}


UriType UriNDEFRecord::getUriType(){
    return uriType;
}



std::string UriNDEFRecord::getPayload(){
    return payload;
}

std::string UriNDEFRecord::getPayloadHex(){
    return charStringToHexString(&payload);
}



NDEFRecordType UriNDEFRecord::getNDEFRecordType(){
    return type;
}

std::string UriNDEFRecord::getNDEFRecordHeader(){
    std::string header;
    header.append("01"); //Type Length URI is 1
    if (payload.length()<200) {
        //add payloadlength Short Record (8 Bit)
        header.append(byteToHexString(payload.length()+1)); 
    }else{
        //Long Record 
        //first to byte 00
        header.append("0000");
        header.append(mifare_desfire::intTo4HexString((int)payload.length()+1));
    }
    //ID length ignored
    header.append("55"); // Type 55 = URI
    header.append(byteToHexString(uriType)); 
    return header;
}

UriNDEFRecord* UriNDEFRecord::parseUriNDEFRecord(std::string urirecord,bool isShort){
    //get size of payload and uri type
    int readPos=0;
    int size=0;
    if (isShort) {
        size = mifare_desfire::hexStringToInt(urirecord.substr(4,2));
        readPos=8;
    }else{
        size = mifare_desfire::hexStringToInt(urirecord.substr(4,8));
        readPos =14;
    }
     size = (size -1)*2;
     UriType uri_type = (UriType)mifare_desfire::hexStringToInt(urirecord.substr(readPos,2));
    readPos+=2;
    std::string payload = mifare_desfire::hexStringToCharString(urirecord.substr(readPos,size));
    return new UriNDEFRecord(uri_type, payload);
   
}